# This Build.PL for Systemd-Daemon was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.014.
use strict;
use warnings;

# inserted by Dist::Zilla::Plugin::CheckLib 0.006
use Devel::CheckLib;
check_lib_or_exit(
    lib => 'systemd',
);

use 5.008;
use Module::Build::Tiny 0.034;
Build_PL();
